-- +----------------------------+     
-- |   Created and Produced by  |     
-- |    Hollow Horse Software   |
-- |          (C) 1997          |
-- +----------------------------+     
----------------------------------------------------------------------------
include sort.e
--global function sort(sequence s)
  --return s
--end function

include rle.e
--global function SPT_to_RLE(sequence s)
  --return s
--end function

--global function RLE_to_SPT(sequence s)
  --return s
--end function

----------------------------------------------------------------------------
--Load palettes & bitmaps
----------------------------------------------------------------------------

global function BMP_load_all_0_0(integer h, integer b)
  integer l, w
  sequence all

  all = repeat(repeat(0, 3), 256)--setup all for holding bitmaps
  all = repeat({all, 0}, b)--setup all for holding bitmaps

  for A = 1 to b do--load all bitmaps
    l = getc(h) + getc(h) * 256--get length or height of bitmap
    w = getc(h) + getc(h) * 256--get width of bitmap

    for B = 1 to 256 by 4 do
      all[A][1][B] = getc(h) & getc(h) & getc(h)
      all[A][1][B + 1] = getc(h) & getc(h) & getc(h)
      all[A][1][B + 2] = getc(h) & getc(h) & getc(h)
      all[A][1][B + 3] = getc(h) & getc(h) & getc(h)
    end for

    all[A][2] = repeat(repeat(0, w), l)--setup up for bitmap
    for B = 1 to l do--load bitmap from file
      for C = 1 to w do
	all[A][2][B][C] = getc(h)
      end for
    end for
  end for

  close(h)
  return all
end function
----------------------------------------------------------------------------
--Load palettes & RLE compressed bitmaps
----------------------------------------------------------------------------

global function BMP_load_all_1_0(integer h, integer b)
  atom s
  integer l, w
  sequence all, hold

  all = repeat(repeat(0, 3), 256)--setup all for holding bitmaps
  all = repeat({all, 0}, b)--setup all for holding bitmaps

  for A = 1 to b do--load all bitmaps
    s = getc(h) + getc(h) * 256
     + getc(h) * 65536 + getc(h) * 16777216--get length of RLE bitmap.
    l = getc(h) + getc(h) * 256--get length or height of bitmap
    w = getc(h) + getc(h) * 256--get width of bitmap

    for B = 1 to 256 by 4 do
      all[A][1][B] = getc(h) & getc(h) & getc(h)
      all[A][1][B + 1] = getc(h) & getc(h) & getc(h)
      all[A][1][B + 2] = getc(h) & getc(h) & getc(h)
      all[A][1][B + 3] = getc(h) & getc(h) & getc(h)
    end for

    hold = repeat(0, s)
    for B = 1 to s do
      hold[B] = getc(h)
    end for
    hold = RLE_to_SPT(hold)

    s = 1
    all[A][2] = repeat(0, l)--setup up for bitmap
    for B = 1 to l do--load bitmap from file
      all[A][2][B] = hold[s..s + w - 1]
      s = s + w
    end for
  end for

  close(h)
  return all
end function
----------------------------------------------------------------------------
--Load RLE compressed palettes & bitmaps
----------------------------------------------------------------------------

global function BMP_load_all_1_1(integer h, integer b)
  atom s
  integer l, w
  sequence all, hold

  all = repeat(repeat(0, 3), 256)--setup all for holding bitmaps
  all = repeat({all, 0}, b)--setup all for holding bitmaps

  for A = 1 to b do--load all bitmaps
    s = getc(h) + getc(h) * 256
     + getc(h) * 65536 + getc(h) * 16777216--get length of RLE bitmap.
    l = getc(h) + getc(h) * 256--get length or height of bitmap
    w = getc(h) + getc(h) * 256--get width of bitmap

    hold = repeat(0, s)
    for B = 1 to s do
      hold[B] = getc(h)
    end for
    hold = RLE_to_SPT(hold)

    s = 1
    for B = 1 to 256 by 2 do
      all[A][1][B] = hold[s..s + 2]
      s = s + 3
      all[A][1][B + 1] = hold[s..s + 2]
      s = s + 3
    end for

    all[A][2] = repeat(0, l)--setup up for bitmap
    for B = 1 to l do--load bitmap from file
      all[A][2][B] = hold[s..s + w - 1]
      s = s + w
    end for
  end for

  close(h)
  return all
end function
----------------------------------------------------------------------------
--Loads all palettes & bitmaps
--This is a version switch box.
----------------------------------------------------------------------------

global function BMP_load_all(sequence file)
  integer h, b
  sequence ver

  h = open(file, "rb")
  if h = -1 then
    puts(2, "Failed to open " & file)
    abort(0)
  end if

  b = getc(h) + getc(h) * 256--get number of bitmaps

  ver = {}
  for A = 1 to 5 do
    ver = append(ver, getc(h))
  end for
  if compare(ver[3..5], "BMP") = 0 then
    ver = ver[1..2]
  else
    puts(2, "UnSupported file type")
    abort(0)
  end if

  if compare(ver, {0, 0}) = 0 then
    return BMP_load_all_0_0(h, b)
  elsif compare(ver, {1, 0}) = 0 then
    return BMP_load_all_1_0(h, b)
  elsif compare(ver, {1, 1}) = 0 then
    return BMP_load_all_1_1(h, b)
  else
    puts(2, "UnSupported file version ")
    ? ver
    abort(0)
  end if

end function
----------------------------------------------------------------------------
--Load palettes & bitmaps
----------------------------------------------------------------------------

global function BMP_load_some_0_0(integer h, integer b, sequence sorted)
  atom c
  integer l, w, n
  sequence some, passed

  c = 7--cnt of where in file we are
  n = 0--keeps value of where to load into some
  some = repeat(repeat(0, 3), 256)--setup some for holding bitmaps
  some = repeat({0, some, 0}, length(sorted))--setup some for holding bitmaps

  passed = {}

  for A = 1 to sorted[length(sorted)] do--load some bitmaps
    if A > b then
      exit
    end if

    passed = passed & A
    l = getc(h) + getc(h) * 256--get length or height of bitmap
    w = getc(h) + getc(h) * 256--get width of bitmap

    c = c + 772 + l * w
    if find(A, sorted) then
      n = n + 1
      for B = 1 to 256 by 4 do
	some[n][2][B] = getc(h) & getc(h) & getc(h)
	some[n][2][B + 1] = getc(h) & getc(h) & getc(h)
	some[n][2][B + 2] = getc(h) & getc(h) & getc(h)
	some[n][2][B + 3] = getc(h) & getc(h) & getc(h)
      end for

      some[n][3] = repeat(repeat(0, w), l)--setup up for bitmap
      for B = 1 to l do--load bitmap from file
	for C = 1 to w do
	  some[n][3][B][C] = getc(h)
	end for
      end for
    elsif machine_func(19, {h, c}) then--seek to pos c
      puts(2, "Seek failed in file ")
      abort(0)
    end if
  end for

  return {passed, some}
end function
----------------------------------------------------------------------------
--Load palettes & RLE compressed bitmaps
----------------------------------------------------------------------------

global function BMP_load_some_1_0(integer h, integer b, sequence sorted)
  atom c, s
  integer l, w, n
  sequence some, passed, hold

  c = 7--cnt of where in file we are
  n = 0--keeps value of where to load into some
  some = repeat(repeat(0, 3), 256)--setup some for holding bitmaps
  some = repeat({0, some, 0}, length(sorted))--setup some for holding bitmaps

  passed = {}

  for A = 1 to sorted[length(sorted)] do--load some bitmaps
    if A > b then
      exit
    end if

    passed = passed & A
    s = getc(h) + getc(h) * 256
     + getc(h) * 65536 + getc(h) * 16777216--get length of RLE bitmap.
    l = getc(h) + getc(h) * 256--get length or height of bitmap
    w = getc(h) + getc(h) * 256--get width of bitmap

    c = c + 776 + s
    if find(A, sorted) then
      n = n + 1
      for B = 1 to 256 by 4 do
	some[n][2][B] = getc(h) & getc(h) & getc(h)
	some[n][2][B + 1] = getc(h) & getc(h) & getc(h)
	some[n][2][B + 2] = getc(h) & getc(h) & getc(h)
	some[n][2][B + 3] = getc(h) & getc(h) & getc(h)
      end for

      hold = repeat(0, s)
      for B = 1 to s do
	hold[B] = getc(h)
      end for
      hold = RLE_to_SPT(hold)

      s = 1
      some[n][3] = repeat(repeat(0, w), l)--setup up for bitmap
      for B = 1 to l do--load bitmap from file
	some[n][3][B] = hold[s..s + w - 1]
	s = s + w
      end for
    elsif machine_func(19, {h, c}) then--seek to pos c
      puts(2, "Seek failed in file ")
      abort(0)
    end if
  end for

  return {passed, some}
end function
----------------------------------------------------------------------------
--Load RLE compressed palettes & bitmaps
----------------------------------------------------------------------------

global function BMP_load_some_1_1(integer h, integer b, sequence sorted)
  atom c, s
  integer l, w, n
  sequence some, passed, hold

  c = 7--cnt of where in file we are
  n = 0--keeps value of where to load into some
  some = repeat(repeat(0, 3), 256)--setup some for holding bitmaps
  some = repeat({0, some, 0}, length(sorted))--setup some for holding bitmaps

  passed = {}

  for A = 1 to sorted[length(sorted)] do--load some bitmaps
    if A > b then
      exit
    end if

    passed = passed & A
    s = getc(h) + getc(h) * 256
     + getc(h) * 65536 + getc(h) * 16777216--get length of RLE bitmap.
    l = getc(h) + getc(h) * 256--get length or height of bitmap
    w = getc(h) + getc(h) * 256--get width of bitmap

    c = c + 8 + s
    if find(A, sorted) then
      n = n + 1

      hold = repeat(0, s)
      for B = 1 to s do
	hold[B] = getc(h)
      end for
      hold = RLE_to_SPT(hold)

      s = 1
      for B = 1 to 256 by 2 do
	some[n][2][B] = hold[s..s + 2]
	s = s + 3
	some[n][2][B + 1] = hold[s..s + 2]
	s = s + 3
      end for

      some[n][3] = repeat(repeat(0, w), l)--setup up for bitmap
      for B = 1 to l do--load bitmap from file
	some[n][3][B] = hold[s..s + w - 1]
	s = s + w
      end for
    elsif machine_func(19, {h, c}) then--seek to pos c
      puts(2, "Seek failed in file ")
      abort(0)
    end if
  end for

  return {passed, some}
end function
----------------------------------------------------------------------------
--Loads some palettes & bitmaps using numberlist
--This is a version switch box.
----------------------------------------------------------------------------

global function BMP_load_some(sequence file, sequence numberlist)
  integer h, b, n
  sequence sorted, some, ver

  sorted = sort(numberlist)

  h = open(file, "rb")
  if h = -1 then
    puts(2, "Failed to open " & file)
    abort(0)
  end if

  b = getc(h) + getc(h) * 256--get number of bitmaps

  ver = {}
  for A = 1 to 5 do
    ver = append(ver, getc(h))
  end for
  if compare(ver[3..5], "BMP") = 0 then
    ver = ver[1..2]
  else
    puts(2, "UnSupported file type")
    abort(0)
  end if

  if compare(ver, {0, 0}) = 0 then
    ver = BMP_load_some_0_0(h, b, sorted)--{passed, some}
  elsif compare(ver, {1, 0}) = 0 then
    ver = BMP_load_some_1_0(h, b, sorted)--{passed, some}
  elsif compare(ver, {1, 1}) = 0 then
    ver = BMP_load_some_1_1(h, b, sorted)--{passed, some}
  else
    puts(2, "UnSupported file version ")
    ? ver
    abort(0)
  end if

  some = ver[2]
  ver = ver[1]--numbers that passed
  if compare(ver, sorted) then
    n = 1
    while n > length(numberlist) do
      if find(numberlist[n], ver) then
	n = n + 1
      else
	numberlist = numberlist[1..n - 1] & numberlist[1..n + 1]
      end if
    end while
  end if
  for A = 1 to length(sorted) do
    some[A][1] = find(ver[A], numberlist)
  end for
  some = sort(some)
  for A = 1 to length(some) do
    some[A] = some[A][2..3]
  end for

  return {numberlist, some}
end function
----------------------------------------------------------------------------

-- In Euphoria 2.1 this file (unless modified) is FREE - 0 statements.
with 736484051 -- delete this statement if it causes an error

